#! /bin/bash

# Author: Francesco Visin
# Version: 1.4
# 
# This script will create a 'package_name'.launch and a launcher file. The first is a standard ros launch file, the second one is another bash script that will setup the environment and then perform the operations necessary to launch the package in gazebo. Finally, the script will look for sdf files and copy them in the right folder to be seen from Gazebo. Please note that it will overwrite existing models named as the folder it is executed in (package_name)
# NOTE: The script has to be copied insied the package folder before being executed.


##### VARIABLES: to be modified #####
# Model specific variables, please modify them with the correct information
create_launch=0                   # if 1, it will look for ONE .world file in the current directory and it will create a .launch file to launch gazebo with it.
create_launcher=0                 # if 1, a launcher file will be created in the current directory
launcher_custom_launch=0          # if 1, it will look for ONE .launch file in the current directory and it will setup the launcher to launch it
                                  # if not 1, or it is unable to find a .launch file, it will launch an empty world
launcher_custom_model=0           # if 1, it will look for ONE .sdf file in the current directory and it will setup the launcher to open a terminal tab 
                                  # with the instructions to spawn the custom model. If not 1 or there is no .sdf file, it will create the normal launcher
create_global_model=0             # if 1, a model will be created in the overall Gazebo's model DB (available from GUI)
model_author="Francesco Visin"
model_author_email="visin@elet.polimi.it"
model_description="Just another model"

##### CONSTANTS: do __NOT__ modify #####
package_name=${PWD##*/}
model_dir=`echo ~`"/.gazebo/models"




##### CODE #####
clear
##### .launch file #####
if [ $create_launch == 1 ]; then
    
  # creating the .launch file
  echo "Creating the .launch file"

  # check the number of world files
  world_nums=`find . -iregex '.*\(world\)' -printf '%f\n' | wc -l`
  if [ $world_nums == 0 ]; then
    echo "ERROR!!"
    echo "One .world file is needed to create the .launcher file! Please leave only one .world file or unflag create_launch and execute the script again"
    exit
  elif [ $world_nums == 1 ]; then
    world_name=`find . -iregex '.*\(world\)' -printf '%f\n'`
  else
    echo "ERROR!!"
    echo "Too many .world files, please leave only one .world file or unflag create_launch and execute the script again"
    exit
  fi

  # generate the .launch file
  echo \
"<launch>

  <arg name=\"paused\" default=\"false\"/>
  <arg name=\"use_sim_time\" default=\"true\"/>
  <arg name=\"gui\" default=\"true\"/>
  <arg name=\"throttled\" default=\"false\"/>

  <!-- set use_sim_time flag -->
  <group if=\"\$(arg use_sim_time)\">
    <param name=\"/use_sim_time\" value=\"true\" />
  </group>


  <!-- start empty world -->
  <group if=\"\$(arg paused)\">
    <group if=\"\$(arg throttled)\">
      <node name=\"gazebo\" pkg=\"gazebo\" type=\"gazebo\" args=\"-u `pwd`/`echo $world_name`\" respawn=\"false\" output=\"screen\"/>
    </group>
    <group unless=\"\$(arg throttled)\">
      <node name=\"gazebo\" pkg=\"gazebo\" type=\"gazebo\" args=\"-u `pwd`/`echo $world_name`\" respawn=\"false\" output=\"screen\"/>
    </group>
  </group>
  <group unless=\"\$(arg paused)\">
    <group if=\"\$(arg throttled)\">
      <node name=\"gazebo\" pkg=\"gazebo\" type=\"gazebo\" args=\"`pwd`/`echo $world_name`\" respawn=\"false\" output=\"screen\"/>
    </group>
    <group unless=\"\$(arg throttled)\">
      <node name=\"gazebo\" pkg=\"gazebo\" type=\"gazebo\" args=\"`pwd`/`echo $world_name`\" respawn=\"false\" output=\"screen\"/>
    </group>
  </group>

  <!-- start gui -->
  <group if=\"\$(arg gui)\">
    <node name=\"gazebo_gui\" pkg=\"gazebo\" type=\"gui\" respawn=\"false\" output=\"screen\"/>
  </group>

</launch>" \
   > `echo $package_name`.launch
   # make it executable
   chmod ugoa+x `echo $package_name`.launch
fi






##### launcher file #####
if [ $create_launcher == 1 ]; then
  # creating launcher
  echo "Creating the launcher"

  # should use custom .launch file?
  if [ $launcher_custom_launch == 1 ]; then
    # check the number of world files
    launch_num=`find . -iregex '.*\(launch\)' -printf '%f\n' | wc -l`
    # if 0 open empty world
    if [ $launch_num == 0 ]; then
      echo "No custom launch file found. Will ignore the launcher_custom_launch flag"
      launch_string="roslaunch gazebo_worlds empty_world.launch;"
    elif [ $launch_num == 1 ]; then
      echo "One custom launch file found: that's ok!"
      launch_name=`find . -iregex '.*\(launch\)' -printf '%f\n'`
      launch_string="roslaunch `echo $launch_name`;"
    else
      echo "ERROR!!"
      echo "Too many .launch files, please unflag launcher_custom_launch or leave zero or one .launch file and execute the script again"
      exit
    fi
  else
    # open empty world
    launch_string="roslaunch gazebo_worlds empty_world.launch;"
  fi

  # should create a tab for the custom model?
  if [ $launcher_custom_model == 1 ]; then
    # check the number of model files
    sdf_num=`find . -iregex '.*\(sdf\)' -printf '%f\n' | wc -l`
    # if zero, ignore the flag
    if [ $sdf_num == 0 ]; then
      echo "No custom model found. Will ignore the launcher_custom_model flag"
    elif [ $sdf_num == 1 ]; then
      echo "One custom model found: that's ok!"
      sdf_name=`find . -iregex '.*\(sdf\)' -printf '%f\n'`
    else
      echo "ERROR!!"
      echo "Too many model (.sdf) files, please unflag launcher_custom_model or leave zero or one .sdf file and execute the script again"
      exit
    fi
  fi

  # generate the launcher file
  echo -n \
"#! /bin/bash

source /opt/ros/groovy/setup.bash
export ROS_PACKAGE_PATH=`pwd`:\$ROS_PACKAGE_PATH
export GAZEBO_PLUGIN_PATH=`pwd`/build:\$GAZEBO_PLUGIN_PATH

echo \"killing ros and gazebo processes, if any\"
killall roscore
killall roslog
killall gzserver

gnome-terminal \\
--tab --title='ROSCORE' -e \"bash -c \\\"echo 'roscore'; export ROS_PACKAGE_PATH=`pwd`:\$ROS_PACKAGE_PATH; roscore; exec bash\\\"\" \\
\\
--tab --title='GAZEBO' -e \"bash -c \\\"echo '$launch_string'; echo 'waiting for roscore...'; sleep 4; export ROS_PACKAGE_PATH=`pwd`:\$ROS_PACKAGE_PATH; export GAZEBO_PLUGIN_PATH=`pwd`/build:\$GAZEBO_PLUGIN_PATH; \\
    $launch_string  exec bash\\\"\"" \
  > launcher

  # add the last part
  if [ $launcher_custom_model == 1 ] && [ $sdf_num == 1 ]; then
    echo " \\
\\
--tab --title='GAZEBO' -e \"bash -c \\\"echo 'rosrun gazebo spawn_model -file `pwd`/`echo $sdf_name` -gazebo -model `echo $package_name` -gazebo_namespace gazeborver'; export ROS_PACKAGE_PATH=`pwd`:\$ROS_PACKAGE_PATH; export GAZEBO_PLUGIN_PATH=`pwd`/build:\$GAZEBO_PLUGIN_PATH; exec bash\\\"\"

exit" >> launcher
  else
    echo "
    
exit" >> launcher
  fi
  
  # make the launcher executable
  chmod ugoa+x launcher

fi





##### model database #####
if [ $create_global_model == 1 ]; then
  # creating or updating the model database
  echo "Adding the model to the DB"
  shopt -s nullglob
  # count sdf files
  sdf_num=`find . -iregex '.*\(sdf\)' -printf '%f\n' | wc -l`

  # exit if there are no sdf files
  if [ $sdf_num == 0 ]; then
    echo "No sdf files found"
    exit  
  fi

  # exit if there is more than one sdf file
  if [ $sdf_num != 1 ]; then
      echo "ERROR: Too many sdf files!!"
      echo "This script assumes that there is only one sdf file, since it will create a new model (only one!) with the same name as the package"
      echo "Please save only one sdf file in the package directory or add the model to the model library manually"
      exit
  fi

  # get sdf filename
  filename=`find . -iregex '.*\(sdf\)' -printf '%f\n'`
  # cut away the full path, if any
  #filename=$(basename "$filename")
  # get only the filename
  filename="${filename%.*}"

  # rename if needed
  if [ "$filename" != "model-1_3" ]; then
      echo "filename is not model-1_3.sdf ---> renaming"
      mv "$filename.sdf" "model-1_3.sdf"
      filename="model-1_3.sdf"
  fi

  # create the model's directory in the model database
  if [[ ! -e "$model_dir/$package_name" ]]; then
      mkdir "$model_dir/$package_name"
  fi

  # create the manifest file
  echo \
"<?xml version=\"1.0\"?>

<model>
  <name>$package_name</name>
  <version>1.0</version>
  <sdf version=\"1.3\">model-1_3.sdf</sdf>

  <author>
    <name>$model_author</name>
    <email>$model_author_email</email>
  </author>

  <description>
    $model_description
  </description>
</model>" \
  > "$model_dir/$package_name/manifest.xml"

  # create the model.config file
  cp "$model_dir/$package_name/manifest.xml" "$model_dir/$package_name/model.config"

  # copy the model-1_3.sdf file
  cp "model-1_3.sdf" "$model_dir/$package_name/model-1_3.sdf"
fi
